@@ -1,24 +1,10 @@
 import sys
 
 PY3 = sys.version_info[0] >= 3
-
-try:
-    from collections import OrderedDict
-except ImportError:  # pragma: no cover
-    from ordereddict import OrderedDict
-
-# pep 8 friendlyness
-OrderedDict
-
+PY2 = not PY3
 
 if PY3:  # pragma: no cover
 
-    def b(s):
-        return s.encode("latin-1")
-
-    def u(s):
-        return s
-
     string_types = str,
     text_type = str
     binary_type = bytes
@@ -36,21 +22,15 @@ if PY3:  # pragma: no cover
 
 else:  # pragma: no cover
 
-    def b(s):
-        return s
-
-    def u(s):
-        return unicode(s, "unicode_escape")
-
-    string_types = basestring,
-    text_type = unicode
-    binary_type = str
-    integer_types = (int, long)
+    string_types = (basestring, )   # noqa: F821
+    text_type = unicode             # noqa: F821
+    binary_type = str               # noqa: F821
+    integer_types = (int, long)     # noqa: F821
 
     def non_native_string(x):
-        if isinstance(x, unicode):
+        if isinstance(x, unicode):  # noqa: F821
             return x
-        return unicode(x, 'unicode_escape')
+        return unicode(x, 'unicode_escape')  # noqa: F821
 
     def make_binary(x):
         if isinstance(x, str):
